/*
Copyright (c) 2012, Felipe Michels Fontoura, Lucas Longen Gioppo,
Luís Guilherme Bergamini Mendes e Renato Girardi Gasoto.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of Universidade Tecnológica Federal do Paraná nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package utfpr.sistemaestagios.dao.text;

import java.io.File;

import utfpr.sistemaestagios.dao.*;

/**
 * @author Felipe Michels Fontoura
 */
public class TextDAOFactory extends DAOFactory {
	private static TextDAOFactory instance;
	private File folder = new File("C:/SERVER/");

	private DAOAluno daoAluno;
	private DAOEmpresa daoEmpresa;
	private DAOProfessor daoProfessor;
	private DAOCoordenador daoCoordenador;
	private DAOEstagio daoEstagio;

	private TextDAOFactory() {
		if (! folder.exists())
			folder.mkdirs();
		daoAluno = new TextDAOAluno(folder); 
		daoEmpresa = new TextDAOEmpresa(folder);
		daoProfessor = new TextDAOProfessor(folder);
		daoCoordenador = new TextDAOCoordenador(folder);
		daoEstagio = new TextDAOEstagio(folder);
	}

	public static DAOFactory getInstance() {
		if (instance == null)
			instance = new TextDAOFactory();
		return instance;
	}

	public DAOAluno getDAOAluno() {
		return daoAluno;
	}

	public DAOEmpresa getDAOEmpresa() {
		return daoEmpresa;
	}

	public DAOProfessor getDAOProfessor() {
		return daoProfessor;
	}

	public DAOCoordenador getDAOCoordenador() {
		return daoCoordenador;
	}

	public DAOEstagio getDAOEstagio() {
		return daoEstagio;
	}
}
