/*
 * Copyright (c) 1996, 2009, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package java.beans;

/**
 * A PropertyVetoException is thrown when a proposed change to a property
 * represents an unacceptable value.
 */

public class PropertyVetoException extends Exception {
	private static final long serialVersionUID = 129596057694162164L;

	/**
	 * Constructs a <code>PropertyVetoException</code> with a detailed message.
	 * 
	 * @param mess
	 *            Descriptive message
	 * @param evt
	 *            A PropertyChangeEvent describing the vetoed change.
	 */
	public PropertyVetoException(String mess, PropertyChangeEvent evt) {
		super(mess);
		this.evt = evt;
	}

	/**
	 * Gets the vetoed <code>PropertyChangeEvent</code>.
	 * 
	 * @return A PropertyChangeEvent describing the vetoed change.
	 */
	public PropertyChangeEvent getPropertyChangeEvent() {
		return evt;
	}

	/**
	 * A PropertyChangeEvent describing the vetoed change.
	 * 
	 * @serial
	 */
	private PropertyChangeEvent evt;
}